#!/bin/bash

# Copyright xmuspeech (Author: Wenxuan Hu, 2021-07-20)
        
# Corresponding email: olr_challenge@163.com
# Please refer to http://olr.cslt.org for more info

. ./path.sh || exit 1;
. ./cmd.sh || exit 1;

# general configuration
backend=pytorch
stage=-1       # start from -1 if you need to start from data download
stop_stage=100
ngpu=8         # number of gpus ("0" uses cpu, otherwise use gpu)
debugmode=1
dumpdir=dump   # directory to dump full features
N=0            # number of minibatches to be used (mainly for debugging). "0" uses all minibatches.
verbose=0      # verbose option
resume=       # Resume the training from snapshot

# feature configuration
do_delta=false

# sample filtering
min_io_delta=4  # samples with `len(input) - len(output) * min_io_ratio < min_io_delta` will be removed.

# config files
preprocess_config=conf/specaug.yaml
train_config=conf/train.yaml
decode_config=conf/decode.yaml

# decoding parameter
recog_model=model.acc.best # set a model to be used for decoding: 'model.acc.best' or 'model.loss.best'
n_average=10

# exp tag
tag="" #tag for managing experiments.

. utils/parse_options.sh || exit 1;

# Set bash to 'debug' mode, it will exit on :
# -e 'error', -u 'undefined variable', -o ... 'error in pipeline', -x 'print commands',
set -e
set -u
set -o pipefail

train_set=train
train_dev=dev

#"data_dir" should be defined as the absolute path of train set.
data_dir=/tsdata/OLR2021
train_name=OLR2021_train

##These are prepared for evaluation sets; "eval_dir" should be defined as the absolute path of test set; "eval_set" should be defined as the filename of test set.
eval_dir=   #/data1/OLR2021/OLR2021_eval/progress_subset
eval_set=   #progress_subset

if [ ${stage} -le 0 ] && [ ${stop_stage} -ge 0 ]; then
    echo "data prep olr"
    # Prepare data sets, all of them contain at least wav.scp, text, spk2utt and utt2spk;
    . ./local/data_train_dev_pre.sh ${data_dir} ${train_name} #In this script you can set your own set of development
    #. ./local/data_eval_pre.sh  ${eval_dir} ${eval_set}
fi

feat_tr_dir=${dumpdir}/${train_set}/delta${do_delta}; mkdir -p ${feat_tr_dir}
feat_dt_dir=${dumpdir}/${train_dev}/delta${do_delta}; mkdir -p ${feat_dt_dir}

if [ ${stage} -le 1 ] && [ ${stop_stage} -ge 1 ]; then
    echo "stage 1: Feature Generation"
    fbankdir=fbank
    for rtask in ${train_name} ${train_dev} ${eval_set}; do
        steps/make_fbank_pitch.sh --cmd "$train_cmd" --nj 32 --write_utt2num_frames true  \
            data/${rtask} exp/make_fbank/${rtask} ${fbankdir}
        utils/fix_data_dir.sh data/${rtask}
    done

    # speed perturb
    utils/perturb_data_dir_speed.sh 0.9 data/${train_name} data/${train_name}_temp1
    utils/perturb_data_dir_speed.sh 1.0 data/${train_name} data/${train_name}_temp2
    utils/perturb_data_dir_speed.sh 1.1 data/${train_name} data/${train_name}_temp3

    utils/combine_data.sh --extra-files utt2uniq data/${train_set} data/${train_name}_temp1 data/${train_name}_temp2 data/${train_name}_temp3
    rm -r data/${train_name}_temp1 data/${train_name}_temp2 data/${train_name}_temp3
    
    steps/make_fbank_pitch.sh --cmd "$train_cmd" --nj 32 --write_utt2num_frames true \
        data/${train_set} exp/make_fbank/${train_set} ${fbankdir}
    utils/fix_data_dir.sh data/${train_set}

    # compute global CMVN
    compute-cmvn-stats scp:data/${train_set}/feats.scp data/${train_set}/cmvn.ark

    # dump features for training
    dump.sh --cmd "$train_cmd" --nj 32 --do_delta ${do_delta} \
        data/${train_set}/feats.scp data/${train_set}/cmvn.ark exp/dump_feats/train ${feat_tr_dir}
    dump.sh --cmd "$train_cmd" --nj 32 --do_delta ${do_delta} \
        data/${train_dev}/feats.scp data/${train_set}/cmvn.ark exp/dump_feats/dev ${feat_dt_dir}


    ##These are prepared for evaluation sets

    # for rtask in ${eval_set}; do
    #     feat_recog_dir=${dumpdir}/${rtask}/delta${do_delta}; mkdir -p ${feat_recog_dir}
    #     dump.sh --cmd "$train_cmd" --nj 32 --do_delta ${do_delta} \
    #         data/${rtask}/feats.scp data/${train_set}/cmvn.ark exp/dump_feats/recog/${rtask} \
    #         ${feat_recog_dir}
    # done
fi

dict=data/lang_1char/${train_set}_units.txt
nlsyms=data/lang_1char/non_lang_syms.txt

echo "dictionary: ${dict}"
if [ ${stage} -le 2 ] && [ ${stop_stage} -ge 2 ]; then
    ### Task dependent. You have to check non-linguistic symbols used in the corpus.
    echo "stage 2: Dictionary and Json Data Preparation"
    
    mkdir -p data/lang_1char/
    echo "make a non-linguistic symbol list"
    cat data/train/text data/dev/text  > data/lang_1char/text
    cut -f 2- data/lang_1char/text | grep -o -P '\[.*?\]|\<.*?\>' | sort | uniq > ${nlsyms}
    echo "*" >> ${nlsyms}
    cat ${nlsyms}
    echo "success"
    
    echo "<unk> 1" > ${dict} # <unk> must be 1, 0 will be used for "blank" in CTC
    text2token.py -s 1 -n 1 -l ${nlsyms} data/lang_1char/text | cut -f 2- -d" " | tr " " "\n" \
    | sort | uniq | grep -v -e '^\s*$' | awk '{print $0 " " NR+1}' >> ${dict}
    wc -l ${dict}

    #make json labels
    data2json.sh --cmd "$train_cmd" --nj 80 --feat ${feat_tr_dir}/feats.scp --nlsyms ${nlsyms} \
         data/${train_set} ${dict} > ${feat_tr_dir}/data.json
    data2json.sh --cmd "$train_cmd" --nj 32 --feat ${feat_dt_dir}/feats.scp --nlsyms ${nlsyms} \
         data/${train_dev} ${dict} > ${feat_dt_dir}/data.json
    

    ##These are prepared for evaluation sets
    
    # for rtask in ${eval_set}; do
    #     feat_recog_dir=${dumpdir}/${rtask}/delta${do_delta}
    #     data2json.sh --cmd "$train_cmd" --nj 32 --feat ${feat_recog_dir}/feats.scp \
    #        --nlsyms ${nlsyms} data/${rtask} ${dict} > ${feat_recog_dir}/data.json
    # done
    
fi


if [ -z ${tag} ]; then
    expname=${train_set}_${backend}_$(basename ${train_config%.*})
    if ${do_delta}; then
        expname=${expname}_delta
    fi
else
    expname=${train_set}_${backend}_${tag}
fi
expdir=exp/${expname}
mkdir -p ${expdir}

echo "success,next step is training"

if [ ${stage} -le 3 ] && [ ${stop_stage} -ge 3 ]; then
    echo "stage 3: Network Training"
    ${cuda_cmd} --gpu ${ngpu} ${expdir}/train.log \
        asr_train.py \
        --config ${train_config} \
        --preprocess-conf ${preprocess_config} \
        --ngpu ${ngpu} \
        --backend ${backend} \
        --outdir ${expdir}/results \
        --tensorboard-dir tensorboard/${expname} \
        --debugmode ${debugmode} \
        --dict ${dict} \
        --debugdir ${expdir} \
        --minibatches ${N} \
        --verbose ${verbose} \
        --resume ${resume} \
        --train-json ${feat_tr_dir}/data.json \
        --valid-json ${feat_dt_dir}/data.json
fi

if [ ${stage} -le 4 ] && [ ${stop_stage} -ge 4 ]; then
    echo "stage 4: Decoding"
	. ./path.sh || exit 1;
    nj=16
    if [[ $(get_yaml.py ${train_config} model-module) = *transformer* ]] || \
           [[ $(get_yaml.py ${train_config} model-module) = *conformer* ]] || \
           [[ $(get_yaml.py ${train_config} etype) = transformer ]] || \
           [[ $(get_yaml.py ${train_config} dtype) = transformer ]]; then
        recog_model=model.last${n_average}.avg.best
        average_checkpoints.py --backend ${backend} \
                    --snapshots ${expdir}/results/snapshot.ep.* \
                    --out ${expdir}/results/${recog_model} \
                    --num ${n_average}
    fi

    for rtask in ${train_dev} ${eval_set}; do
    (
        decode_dir=decode_${rtask}_$(basename ${decode_config%.*})
        feat_recog_dir=${dumpdir}/${rtask}/delta${do_delta}

        # split data
        splitjson.py --parts ${nj} ${feat_recog_dir}/data.json

        #### use CPU for decoding
        ngpu=0

        ${decode_cmd} JOB=1:${nj} ${expdir}/${decode_dir}/log/decode.JOB.log \
            asr_recog.py \
            --config ${decode_config} \
            --ngpu ${ngpu} \
            --backend ${backend} \
            --recog-json ${feat_recog_dir}/split${nj}utt/data.JOB.json \
            --result-label ${expdir}/${decode_dir}/data.JOB.json \
            --model ${expdir}/results/${recog_model}

        local/score_sclite.sh --nlsyms ${nlsyms}  ${expdir}/${decode_dir} ${dict}

    ) 
    done

    echo "Finished"
fi
